#ifndef __TSMUXER_H
#define __TSMUXER_H

#include <QtGui>
#include <QFileDialog>
#include <QProcess>

#include "ui_tsmuxerwindow.h"
#include "muxForm.h"

struct QtvCodecInfo
{
	QtvCodecInfo():
		  dtsDownconvert(false),
		  delay(0),
		  trackID(0),
		  width(0),
		  height(0),
		  checkFPS(false),
		  checkLevel(false),
		  addSEI(true),
		  delPulldown(-1),
		  addSPS(true),
		  nested(0),
		  parent(0),
		  child(0),
		  bindFps(true) {}

	int trackID;
	int width;
	int height;
	QString displayName;
	QString programName;
	QString descr;
	QString lang;
	int delay;
	bool dtsDownconvert;
	QList<QString> fileList;
	bool checkFPS;
	bool checkLevel;
	bool addSEI;
	bool addSPS;
	int delPulldown;
	QString fpsText;
	QString fpsTextOrig;
	QString levelText;
	// for append
	int nested;
	QtvCodecInfo* parent;
	QtvCodecInfo* child;
	bool bindFps;
	QString mplsFile;
	QString arText;
};

class TsMuxerWindow: public QWidget
{
    Q_OBJECT
public:
    TsMuxerWindow();
	~TsMuxerWindow();
signals:
	void tsMuxerSuccessFinished();
	void codecListReady();
	void fileAdded();
	void fileAppended();
private slots:
	void onAddBtnClick();
	void readFromStdout();
	void readFromStderr();
	void onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus);
	void onProcessError(QProcess::ProcessError error);
	void trackLVItemSelectionChanged();
	void inputFilesLVChanged();
	void trackLVItemChanged(QTableWidgetItem* item);
	void onVideoCheckBoxChanged(int state);
	void onPulldownCheckBoxChanged(int state);
	void onVideoComboBoxChanged(int index);
	void onAudioSubtitlesCheckBoxChanged(int state);
	void onAudioSubtitlesComboBoxChanged(int index);
	void onEditDelayChanged(int i);
	void onFontBtnClicked();
	void onColorBtnClicked();
	void onGeneralCheckboxClicked();
	void onGeneralSpinboxValueChanged();
	void onChapterParamsChanged();
	void onSplitCutParamsChanged();
	void onFontParamsChanged();
	void onRemoveBtnClick();
	void onAppendButtonClick();
	void onRemoveTrackButtonClick();
	void onMoveUpButtonCLick();
	void onMoveDownButtonCLick();
	void RadioButtonMuxClick();
	void outFileNameChanged();
	void saveFileDialog();
	void startMuxing();
	void saveMetaFileBtnClick();
	void continueAppendFile();
	void continueAddFile();
	void getCodecInfo();
	void addFile();
	void appendFile();
	void onOpacityTimer();
protected:
	virtual void closeEvent(QCloseEvent * event );
private:
	bool checkFileDuplicate(const QString& fileName);
	void shellExecute(const QString& process, const QStringList& args);
	void addLines(const QByteArray& arr, QList<QString>& outList, bool isError);
	void doAppendInt(const QString& fileName, const QString& parentFileName);
	void updateMetaLines();
	QString getMuxOpts();
	QString getAudioMetaInfo(QtvCodecInfo* codecInfo);
	QString getVideoMetaInfo(QtvCodecInfo* codecInfo);
	QString getSrtParams();
	int findLangByCode(const QString& code);
	void setComboBoxText(QComboBox* comboBox, const QString& text);
	QtvCodecInfo* getCurrentCodec();
	void delTracksByFileName(const QString& fileName);
	void deleteTrack(int idx);
	void updateNum();
	void modifyOutFileName(const QString fileName);
	void moveRow(int index, int index2);
	bool saveMetaFile(const QString& metaName);
	void updateBtns(QWidget* w);

	//QTemporaryFile* tempFile;
	//QString tempFileName;
	QString metaName;
	Ui::TsMuxerWindow ui;
	QFileDialog* openFileDialog;
	bool trackLVIngoreOnChange;
	bool processFinished;
	int processExitCode;
	QProcess::ExitStatus processExitStatus;
	QProcess::ProcessError processError;
    QList<QString> mplsFileList;
	QProcess proc;
	QList<QString> procStdOutput;
	QList<QString> procErrOutput;
	bool outFileNameModified;
	QString oldFileName;
	bool outFileNameDisableChange;
	QString saveDialogFilter;
	MuxForm muxForm;
	QString newFileName;
	QList<QtvCodecInfo> codecList;
	bool runInMuxMode;
	QString lastSelectedDir;
	QList <QUrl> addFileList;
	QTimer opacityTimer;

	void dragEnterEvent(QDragEnterEvent *event);
	void dropEvent(QDropEvent* event);
	void dragMoveEvent(QDragMoveEvent *event);
	void dragLeaveEvent ( QDragLeaveEvent * event);

	QTemporaryFile* tempSoundFile;
	QSound* sound;
	void myPlaySound(const QString& fileName);
	bool isVideoCropped();
};

#endif
